﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AircraftCarrier
{
    /// <summary>
    /// Класс, отвечающий за прорисовку и перемещение объекта-сущности
    /// </summary>
    internal class DrawingWarship
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityWarship Warship { get; private set; }
        /// <summary>
        /// Левая координата отрисовки военного корабля
        /// </summary>
        private float _startPosX;
        /// <summary>
        /// Верхняя кооридната отрисовки военного корабля
        /// </summary>
        private float _startPosY;
        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private int? _pictureWidth = null;
        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private int? _pictureHeight = null;
        /// <summary>
        /// Ширина отрисовки военного корабля
        /// </summary>
        private readonly int _warshipWidth = 94;
        /// <summary>
        /// Высота отрисовки военного корабля
        /// </summary>
        private readonly int _warshipHeight = 40;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес военного корабля</param>
        /// <param name="bodyColor">Цвет главной палубы</param>
        public void Init(int speed, float weight, Color bodyColor)
        {
            Warship = new EntityWarship();
            Warship.Init(speed, weight, bodyColor);
        }
        /// <summary>
        /// Установка позиции военного корабля
        /// </summary>
        /// <param name="x"></param>
        /// <param name="y"></param>
        /// <param name="width"></param>
        /// <param name="height"></param>
        public void SetPosition(int x, int y, int width, int height)
        {
            if (width >= x + _warshipWidth && height >= y + _warshipHeight && x >= 0 && y >= 0)
            {
                _startPosX = x;
                _startPosY = y;
                _pictureWidth = width;
                _pictureHeight = height;
            }      
        }
        /// <summary>
        /// Изменение направления пермещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(Direction direction)
        {
            if (!_pictureWidth.HasValue || !_pictureHeight.HasValue)
            {
                return;
            }
            switch (direction)
            {
                // вправо
                case Direction.Right:
                    if (_startPosX + _warshipWidth + Warship.Step < _pictureWidth)
                    {
                        _startPosX += Warship.Step;
                    }
                    break;
                //влево
                case Direction.Left:
                    if(_startPosX - Warship.Step > 0)
                    {
                        _startPosX -= Warship.Step;
                    }
                    break;
                //вверх
                case Direction.Up:
                    if (_startPosY - Warship.Step > 0)
                    {
                        _startPosY -= Warship.Step;
                    }
                    break;
                //вниз
                case Direction.Down:
                    if (_startPosY + _warshipHeight + Warship.Step < _pictureHeight)
                    {
                        _startPosY += Warship.Step;
                    }
                    break;
            }
        }

        /// <summary>
        /// Отрисовка военного корабля
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (_startPosX < 0 || _startPosY < 0
            || !_pictureHeight.HasValue || !_pictureWidth.HasValue)
            {
                return;
            }
            Pen pen = new(Color.Black);

            //границы военного корабля
            PointF point1 = new PointF(_startPosX, _startPosY);
            PointF point2 = new PointF(_startPosX + 74, _startPosY);
            PointF point3 = new PointF(_startPosX + 94, _startPosY + 20);
            PointF point4 = new PointF(_startPosX + 74, _startPosY + 40);
            PointF point5 = new PointF(_startPosX, _startPosY + 40);

            PointF[] curvePoints =
            {
                point1, point2, point3, point4, point5
            };
            g.DrawPolygon(pen, curvePoints);

            //главная палуба
            Brush br = new SolidBrush(Warship?.BodyColor ?? Color.White);
            g.FillPolygon(br, curvePoints);

            //мачта
            Brush brWhite = new SolidBrush(Color.White);
            g.FillEllipse(brWhite, _startPosX + 59, _startPosY + 13, 15, 15);

            //границы мачты
            g.DrawEllipse(pen, _startPosX + 59, _startPosY + 13, 15, 15);

            //палуба
            g.FillRectangle(brWhite, _startPosX + 44, _startPosY + 10, 10, 20);
            g.FillRectangle(brWhite, _startPosX + 24, _startPosY + 15, 20, 10);

            //границы палуба
            g.DrawRectangle(pen, _startPosX + 44, _startPosY + 10, 10, 20);
            g.DrawRectangle(pen, _startPosX + 24, _startPosY + 15, 20, 10);

            //двигатели
            Brush brBlack = new SolidBrush(Color.Black);
            g.FillRectangle(brBlack, _startPosX, _startPosY + 5, 4, 10);
            g.FillRectangle(brBlack, _startPosX, _startPosY + 23, 4, 10);
        }

        /// <summary>
        /// Смена границ формы отрисовки
        /// </summary>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        public void ChangeBorders(int width, int height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_pictureWidth <= _warshipWidth || _pictureHeight <= _warshipHeight)
            {
                _pictureWidth = null;
                _pictureHeight = null;
                return;
            }
            if (_startPosX + _warshipWidth > _pictureWidth)
            {
                _startPosX = _pictureWidth.Value - _warshipWidth;
            }
            if (_startPosY + _warshipHeight > _pictureHeight)
            {
                _startPosY = _pictureHeight.Value - _warshipHeight;
            }
        }

    }
}
